package platApi

import (
	"bytes"
	"crypto"
	"crypto/rand"
	"crypto/rsa"
	"crypto/sha256"
	"crypto/x509"
	"encoding/base64"
	"encoding/json"
	"errors"
	"fmt"
	"game/base/log"
	"game/base/util"
	"io/ioutil"
	"net/http"
	"net/url"
	"sort"
	"time"
)

func oppoSign(data map[string]interface{}, private string) (string, error) {
	//STEP 1, 对key进行升序排序.
	sorted_keys := make([]string, 0)
	for k := range data {
		sorted_keys = append(sorted_keys, k)
	}
	sort.Strings(sorted_keys)

	//STEP2, 对key=value的键值对用&连接起来，略过空值
	var signStrings string
	for i, k := range sorted_keys {
		value := fmt.Sprintf("%v", data[k])
		if value != "" {
			if i != (len(sorted_keys) - 1) {
				signStrings = signStrings + k + "=" + value + "&"
			} else {
				signStrings = signStrings + k + "=" + value //最后一个不加此符号
			}
		}
	}
	keyByts, err := base64.StdEncoding.DecodeString(private)
	if err != nil {
		log.Error(err)
		return "", err
	}

	privateKey, err := x509.ParsePKCS8PrivateKey(keyByts)
	if err != nil {
		log.Error("ParsePKCS8PrivateKey err", err)
		return "", err
	}
	h := sha256.New()
	h.Write([]byte(signStrings))
	signature, err := rsa.SignPKCS1v15(rand.Reader, privateKey.(*rsa.PrivateKey), crypto.SHA256, h.Sum(nil))
	if err != nil {
		log.Errorf("Error from signing: %s\n", err)
		return "", err
	}
	return base64.StdEncoding.EncodeToString(signature), nil
}

type OrderData struct {
	AppId     string `json:"appId"`
	CpOrderId string `json:"cpOrderId"`
	OrderNo   string `json:"orderNo"`
}

type oppoPreOrderRet struct {
	Code string    `json:"code"`
	Msg  string    `json:"msg"`
	Data OrderData `json:"data"`
}

type oppoPreOrderReq struct {
	AppId         string `json:"appId"`
	OpenId        string `json:"openId"`
	Timestamp     int64  `json:"timestamp"`
	Sign          string `json:"sign"`
	ProductName   string `json:"productName"`
	ProductDesc   string `json:"productDesc"`
	Count         int64  `json:"count"`
	Price         int64  `json:"price"`
	Currency      string `json:"currency"`
	CallBackUrl   string `json:"callBackUrl"`
	CpOrderId     string `json:"cpOrderId"`
	AppVersion    string `json:"appVersion"`
	EngineVersion string `json:"engineVersion"`
	Attach        string `json:"attach"`
}
type OppoPreOrderResult struct {
	Order     string `json:"order"`
	Timestamp int64  `json:"timestamp"`
	CliSign   string `json:"clisign"`
}

func OppoReqOrder(appID string, appKey string, openId, productName, productDesc, appVersion, engineVersion string, count, price int64, callback string, privateKey string, attach string) (*OppoPreOrderResult, error) {
	orderID, err := util.NextId()
	if err != nil {
		return nil, err
	}

	timestamp := time.Now().UnixNano() / 1e6
	cpOrderId := util.Int64ToString(orderID)
	m := make(map[string]interface{})
	m["appId"] = appID
	m["appVersion"] = appVersion
	m["callBackUrl"] = callback
	m["count"] = count
	m["cpOrderId"] = cpOrderId
	m["currency"] = "CNY"
	m["engineVersion"] = engineVersion
	m["openId"] = openId
	m["price"] = price
	m["productDesc"] = productDesc
	m["productName"] = productName
	m["timestamp"] = timestamp
	m["attach"] = attach

	sign, err := oppoSign(m, privateKey)
	if err != nil {
		return nil, err
	}

	req := &oppoPreOrderReq{
		AppId:         appID,
		OpenId:        openId,
		Timestamp:     timestamp,
		Sign:          sign,
		ProductName:   productName,
		ProductDesc:   productDesc,
		Count:         count,
		Price:         price,
		Currency:      "CNY",
		CallBackUrl:   callback,
		CpOrderId:     cpOrderId,
		AppVersion:    appVersion,
		EngineVersion: engineVersion,
		Attach:        attach,
	}
	strParam, err := json.Marshal(req)
	if err != nil {
		return nil, err
	}

	urlStr := "https://jits.open.oppomobile.com/jitsopen/api/pay/v1.0/preOrder"
	resp, err := http.Post(urlStr, "application/json", bytes.NewBuffer(strParam))
	if err != nil {
		return nil, err
	}
	defer resp.Body.Close()
	body, err := ioutil.ReadAll(resp.Body)
	if err != nil {
		return nil, err
	}

	ret := &oppoPreOrderRet{}
	err = json.Unmarshal(body, ret)
	if err != nil {
		return nil, err
	}
	if ret.Code != "200" {
		err = errors.New(ret.Code + ret.Msg)
		return nil, err
	}
	if ret.Data.CpOrderId != cpOrderId || ret.Data.AppId != appID {
		err = errors.New("Error Param ")
		return nil, err
	}
	cliM := make(map[string]interface{})
	cliM["appKey"] = appKey
	cliM["orderNo"] = ret.Data.OrderNo
	cliM["timestamp"] = timestamp

	cliSign, err := oppoSign(cliM, privateKey)
	if err != nil {
		return nil, err
	}
	r := &OppoPreOrderResult{
		Order:     ret.Data.OrderNo,
		Timestamp: timestamp,
		CliSign:   cliSign,
	}
	return r, nil
}

func OppoCallBackVerify(value url.Values, publicKey string) error {
	attach := value.Get("attach")
	notifyId := value.Get("notifyId")
	partnerOrder := value.Get("partnerOrder")
	paymentWay := value.Get("paymentWay")
	payResult := value.Get("payResult")
	productDesc := value.Get("productDesc")
	productName := value.Get("productName")
	sign := value.Get("sign")
	count := util.StringToUint32(value.Get("count"))
	price := util.StringToUint32(value.Get("price"))

	// 验签
	content := fmt.Sprintf("attach=%s&count=%d&notifyId=%s&partnerOrder=%s&payResult=%s&paymentWay=%s&price=%d&productDesc=%s&productName=%s",
		attach, count, notifyId, partnerOrder, payResult, paymentWay, price, productDesc, productName)

	sig, err := base64.StdEncoding.DecodeString(sign)
	if err != nil {
		return err
	}
	public, _ := base64.StdEncoding.DecodeString(publicKey)
	pub, err := x509.ParsePKIXPublicKey(public)
	if err != nil {
		return err
	}
	hash := sha256.New()
	hash.Write([]byte(content))
	return rsa.VerifyPKCS1v15(pub.(*rsa.PublicKey), crypto.SHA256, hash.Sum(nil), sig)
}
